@@ -44,6 +44,7 @@ def main_menu(items, selected, title=None, search=None, white=False,
         'white': bool(white),
         }
 
+
 @register.inclusion_tag('bob/tab_menu.html')
 def tab_menu(items, selected, side=None):
     """
@@ -61,6 +62,7 @@ def tab_menu(items, selected, side=None):
         'side': side,
         }
 
+
 @register.inclusion_tag('bob/sidebar_menu.html')
 def sidebar_menu(items, selected):
     """
@@ -75,6 +77,7 @@ def sidebar_menu(items, selected):
         'selected': selected,
         }
 
+
 @register.inclusion_tag('bob/sidebar_menu_subitems.html')
 def sidebar_menu_subitems(item, selected):
     """
@@ -86,10 +89,11 @@ def sidebar_menu_subitems(item, selected):
         'selected': selected,
         }
 
+
 @register.inclusion_tag('bob/pagination.html')
 def pagination(page, show_all=False, show_csv=False,
                fugue_icons=False, url_query=None, neighbors=1,
-               query_variable_name='page'):
+               query_variable_name='page', export_variable_name='export'):
     """
     Display pagination for a list of items.
 
@@ -138,9 +142,11 @@ def pagination(page, show_all=False, show_csv=False,
         'url_previous_page': changed_url(url_query, query_variable_name, page_no-1),
         'url_next_page': changed_url(url_query, query_variable_name, page_no+1),
         'url_pages': url_pages,
-        'url_all': changed_url(url_query, query_variable_name, 0)
+        'url_all': changed_url(url_query, query_variable_name, 0),
+        'export_variable_name': export_variable_name,
     }
 
+
 def changed_url(query, name, value):
     if not query:
         return '%s=%s' % (name, value)
@@ -154,6 +160,7 @@ def changed_url(query, name, value):
             pass
     return query.urlencode()
 
+
 @register.filter
 def bob_export(query, export):
     """Modify the query string of an URL to change the ``export`` argument."""
@@ -192,25 +199,80 @@ def timesince_limited(d):
 
 
 @register.inclusion_tag('bob/form.html')
-def form(form, action="", method='POST', fugue_icons=False,
-                    css_class="form-horizontal"):
+def form(form, action="", method="POST", fugue_icons=False,
+         css_class="form-horizontal", title="", submit_label='Save'):
     """
-    Render a horizontal form.
+    Render a form.
 
     :param form: The form to render.
     :param action: The submit URL.
     :param method: The submit method, either ``"GET"`` or ``"POST"``.
     :param fugue_icons: Whether to use Fugue or Bootstrap icon.
     :param css_class: The CSS class to use for the ``<form>`` tag.
+    :param title: Form title.
+    :param submit_label: Submit button label.
     """
     return {
         'form': form,
         'action': action,
+        'title': title,
         'method': method,
-        'fugueicons': fugue_icons,
+        'fugue_icons': fugue_icons,
         'css_class': css_class,
+        'submit_label': submit_label,
     }
 
+
 @register.inclusion_tag('bob/form.html')
 def form_horizontal(*args, **kwargs):
     return form(*args, **kwargs)
+
+
+@register.inclusion_tag('bob/table_header.html')
+def table_header(columns=None, url_query=None, sort=None, fugue_icons=False,
+                 sort_variable_name='sort'):
+    """
+    Render a table header with sorted column options
+
+    :param columns: a list of objects of
+    type :py:class:bob.data_table.DataTableColumn
+    :param url_query: The query parameters to add to all page links
+    :param sort: means that the column is now sorted
+    :param fugue_icons: Whether to use Fugue icons or Bootstrap icons.
+    """
+    return {
+        'columns': columns,
+        'sort': sort,
+        'url_query': url_query,
+        'fugue_icons': fugue_icons,
+        'sort_variable_name': sort_variable_name,
+    }
+
+@register.simple_tag
+def bob_sort_url(query, field, sort_variable_name, type):
+    """Modify the query string of an URL to change the ``sort_variable_name``
+    argument.
+    """
+    query = query.copy()
+    if type == 'desc':
+        query[sort_variable_name] = '-' + field
+    elif type == 'asc':
+        query[sort_variable_name] = field
+    return query.urlencode()
+
+@register.simple_tag
+def bob_export_url(query, value, export_variable_name='export'):
+    """Modify the query string of an URL to change the ``export_variable_name``
+    argument.
+    """
+    if not query:
+        return '%s=%s' % (export_variable_name, value)
+    query = query.copy()
+    if value:
+        query[export_variable_name] = value
+    else:
+        try:
+            del query[export_variable_name]
+        except KeyError:
+            pass
+    return query.urlencode()
